% Inverted penmdulum on a cart model from
%
% John Hauser and Hinke Osinga.  On the geometry of optimal control:
%    the inverted pendulum example.  In Proceedings of the American
%    Control Conference, pages 1721--1726, Washington, DC, 2001.

% Comments from Osinga's Research Web-page
% http://www.enm.bris.ac.uk/staff/hinke/optimal/

% We study the geometry of the optimal solution to the problem of balancing an inverted planar
%   pendulum on a cart subject to a quadratic cost function. The cart moves in the plane of the 
%   pendulum with an applied horizontal force u constituting the control. The mass of the cart 
%   is M, the mass of the pendulum m, and its center of mass is at distance l from the pivot.

% Here, x1 is the angle measured from the vertical up position, mr = m / (m + M) is the mass 
%   ratio, and g is the gravitational constant. We do not consider the model equations associated 
%   with the cart. The origin is an unstable equilibrium corresponding to the vertical position 
%   above the pivot.

% Notes from J.D. Hedengren - theta is in radians.  Watch that -pi/2 < theta < pi/2 for physical 
%   constraints.

function xdot = pend(t,x)

global u

% Input (1):
% Horizontal Force (N)
F_hor = u;

% States (2):
% Angle of the Pendulum (radians)
theta=x(1,1);
% Rate of Change of the Angle (rad/sec)
thetadot=x(2,1);

% Parameters
% Gravitational Constant (m/s^2)
g =9.8;
% Mass of the Pendulum (kg)
m=2;
% Mass of the Cart (kg)
M=8;
% Length of the Pendulum (m)
l=.5;
% Mass Ratio
mr = m/(m+M);

% Compute xdot
xdot(1,1) = thetadot;
xdot(2,1) = (g/l * sin(theta) - (mr*thetadot^2)/2 * sin(2*theta) - ...
   mr/(m*l) * F_hor*cos(theta))/(4/3 - mr*(cos(theta))^2);
